=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2006-08-05 22:33:36 +0000
+++ bzrlib/bundle/bundle_data.py	2006-08-08 06:47:16 +0000
@@ -652,6 +652,7 @@
             inv = Inventory(root_id, self.revision_id)
         except TypeError:
             inv = Inventory(revision_id=self.revision_id)
+        inv.root.revision = self.get_last_changed(root_id)
 
         def add_entry(file_id):
             path = self.id2path(file_id)

=== modified file 'bzrlib/bundle/serializer/v08.py'
--- bzrlib/bundle/serializer/v08.py	2006-08-05 22:33:36 +0000
+++ bzrlib/bundle/serializer/v08.py	2006-08-08 06:47:16 +0000
@@ -317,7 +317,7 @@
         self.from_file = iter(from_file)
         self._next_line = None
         
-        self.info = BundleInfo()
+        self.info = BundleInfo08()
         # We put the actual inventory ids in the footer, so that the patch
         # is easier to read for humans.
         # Unfortunately, that means we need to read everything before we
@@ -500,3 +500,8 @@
                 # Consume the trailing \n and stop processing
                 self._next().next()
                 break
+
+class BundleInfo08(BundleInfo):
+    def _update_tree(self, bundle_tree, revision_id):
+        bundle_tree.note_last_changed('', revision_id)
+        BundleInfo._update_tree(self, bundle_tree, revision_id)

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-08-05 22:33:36 +0000
+++ bzrlib/repository.py	2006-08-08 06:47:16 +0000
@@ -443,7 +443,9 @@
         :param revision_id: The expected revision id of the inventory.
         :param xml: A serialised inventory.
         """
-        return xml5.serializer_v5.read_inventory_from_string(xml)
+        result = xml5.serializer_v5.read_inventory_from_string(xml)
+        result.root.revision = revision_id
+        return result
 
     @needs_read_lock
     def get_inventory_xml(self, revision_id):

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-08-08 06:47:16 +0000
@@ -313,6 +313,13 @@
             self.assertEqual(revision.revision_id, revision_id)
             self.assertEqual(revision, repo.get_revision(revision_id))
 
+    def test_root_entry_has_revision(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('message', rev_id='rev_id')
+        self.assertEqual(tree.basis_tree().inventory.root.revision, 'rev_id')
+        rev_tree = tree.branch.repository.revision_tree(tree.last_revision())
+        self.assertEqual(rev_tree.inventory.root.revision, 'rev_id')
+
 
 class TestCaseWithComplexRepository(TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2006-08-04 04:41:02 +0000
+++ bzrlib/tests/test_bundle.py	2006-08-08 06:47:16 +0000
@@ -799,6 +799,14 @@
         self.assertEqual(19800, rev.timezone)
         self.assertEqual(1152544886.0, rev.timestamp)
 
+    def test_bundle_root_id(self):
+        self.tree1 = self.make_branch_and_tree('b1')
+        self.b1 = self.tree1.branch
+        self.tree1.commit('message', rev_id='revid1')
+        bundle = self.get_valid_bundle(None, 'revid1')
+        tree = bundle.revision_tree(self.b1.repository, 'revid1')
+        self.assertEqual(tree.inventory.root.revision, 'revid1')
+
 
 class MungedBundleTester(TestCaseWithTransport):
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-08-08 00:24:38 +0000
+++ bzrlib/workingtree.py	2006-08-08 06:47:16 +0000
@@ -399,6 +399,7 @@
             try:
                 xml = self.read_basis_inventory()
                 inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
+                inv.root.revision = revision_id
             except NoSuchFile:
                 inv = None
             if inv is not None and inv.revision_id == revision_id:

